function [DGP Simu MLL_tab] = generate_posterior_distributions_AR_process(T,AR_lags,nb_MCMC)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Tutorial : Gibbs sampler for AR processes. The program generates an simulated series from an AR process with AR_lags number of lags
%%%%% and carries out MCMC inference as well as the computation of MLLs according to different methods (Chib,Importance Sampling,Harmonic Mean)
%%%%% The output provides the simulated series (in DGP), the MCMC draws
%%%%% from an AR process with one lags up to AR_lags+1 (in Simu) and a
%%%%% matrix summarizes the MLL values of the different models computed
%%%%% by Chib's formula and Importance Sampling.
%%%%% Be careful :  Harmonic mean estimator
%%%%% requires too many draws to converge to the true estimate. Do not use
%%%%% it in practice.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Inputs : 
%%%%% T is the size of the simulated series
%%%%% AR_lags denotes the number of lags in the AR process of the DGP.
%%%%% nb_MCMC : number of MCMC iterations 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Outputs :
%%%%% DGP stands for a structure containing  : 
%%%%% - y : the simulated series
%%%%% - X : the explanatory variables
%%%%% - theta : the mean parameters
%%%%% - sigma_sq : the variance parameter
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Simu is a structure containing :
%%%%% - post_beta and post_sigma : MCMC Posterior draws of Beta and sigma^2
%%%%% under CP priors
%%%%% - beta_ols and sigma_ols are the OLS results.
%%%%% - MLL_Chib : MLL computed from MCMC output
%%%%% - MLL_IS : MLL computed by IS
%%%%% - MLL_HM : MLL computed by Harmonic mean 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% - MLL_tab contains in column the MLL of the different models (from
%%%%% AR_lags = 1 up to AR_lags = AR_lags +1) computed by Chib in column 1
%%%%% and by Importance Sampling (in column 2);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%% By default, set the variable nb_MCMC = 5000.
if(nargin<3)
    nb_MCMC=5000;
end
model_max = AR_lags+1;

[DGP] = generation_AR_process(T,AR_lags);
MLL_tab = zeros(model_max,2);
for q=1:model_max
    [X y] = AR_explanatory_var(DGP.y,q);
    [Simu(q)] = Gibbs_regression_with_MLL(y,X,nb_MCMC); %#ok<AGROW>
    MLL_tab(q,1) = Simu(q).MLL_Chib;
    MLL_tab(q,2) = Simu(q).MLL_IS;
end

disp(' Marginal log-likelihood of the different models : ')
disp(MLL_tab(:,1)');
disp('***************************************************');
[~,ind] = max(MLL_tab(:,1));
disp(['The best model according to MLL is an AR of order : ' num2str(ind)]);