function [sn log_like] = Forward_Backward(y,X,beta,sigma,trans_mat)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% function that applies the forward Backward algorithm for CP models
%%%%% given the parameters and the dependent and explanatory variables.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Inputs : 
%%%%% y contains the dependent variable (T * 1)
%%%%% X stands for the explanatory variables (T * k)
%%%%% Sigma (regime*1) : the variance parameters
%%%%% beta (k*regime) : the mean parameters
%%%%% trans_mat (regime*1) : the diagonal of the transition matrix
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Outputs :
%%%%% sn : a draw of the state vector
%%%%% log_like : The log-likelihood of the model where the state vector has
%%%%% been integrated ( f(Y_T|beta,sigma,trans_mat) )
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Initialisation of the algorithm
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
T = max(size(y));
regime = max(size(trans_mat));
forward = zeros(T,regime);
forward(1,1) = 1;
P = diag(trans_mat);
for k=1:regime-1
   P(k,k+1) = 1-P(k,k);
end
log_like = 0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Forward algorithm
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for t=2:T
    constante = 0;
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%% Predictive step
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    transition = forward(t-1,:)*P;
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%% Update step
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    for k=1:regime
        if(transition(k)~=0)
            forward(t,k) =  -0.5*log(2*pi*sigma(k)) - 0.5*((y(t)-X(t,:)*beta(:,k)))^2/sigma(k) + log(transition(k));
            if(constante==0)
                constante = forward(t,k);
            elseif(constante<forward(t,k))
                constante = forward(t,k);
            end
        end
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%% Step for avoiding numerical problems
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   for k=1:regime
       if(forward(t,k)~=0)
           forward(t,k) = exp(forward(t,k)-constante);
       end
   end
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   %%%% Normalizing the probabilities
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   Q = sum(forward(t,:));
   log_like = log_like + log(Q) + constante;
   forward(t,:) = forward(t,:)/Q;
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Backward algorithm : sampling a state vector
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
sn = zeros(T,1);
sn(T) = regime;
for t=T-1:-1:1
   etat_fut = sn(t+1);
   back = forward(t,:)'.*P(:,etat_fut); 
   back = back/sum(back);
   sn(t) =  multinomialrnd(back);
end

     
